//
// APIResponse.swift
//
// Generated by SwagGen
// https://github.com/yonaskolb/SwagGen
//

import Alamofire

public protocol APIResponseValue: CustomDebugStringConvertible, CustomStringConvertible {
    var statusCode: Int { get }
    var successful: Bool { get }
    var response: Any { get }
    init(statusCode: Int, json: Any) throws
}

public struct APIResponse<T: APIResponseValue> {

    /// The APIRequest used for this response
    public let request: APIRequest<T>

    /// The result of the response .
    public let result: APIResult<T>

    /// The URL request sent to the server.
    public let urlRequest: URLRequest?

    /// The server's response to the URL request.
    public let urlResponse: HTTPURLResponse?

    /// The data returned by the server.
    public let data: Data?

    /// The timeline of the complete lifecycle of the request.
    public let timeline: Timeline?

    init(request: APIRequest<T>, result: APIResult<T>, urlRequest: URLRequest? = nil, urlResponse: HTTPURLResponse? = nil, data: Data? = nil, timeline: Timeline? = nil) {
        self.request = request
        self.result = result
        self.urlRequest = urlRequest
        self.urlResponse = urlResponse
        self.data = data
        self.timeline = timeline
    }
}

extension APIResponse: CustomStringConvertible, CustomDebugStringConvertible {

    public var description:String {
        var string = "\(request)"

        switch result {
        case .success(let value):
            string += " returned \(value.statusCode)"
            let responseString = "\(type(of: value.response))"
            if responseString != "()" {
                string += ": \(responseString)"
            }
        case .failure(let error): string += " failed: \(error)"
        }
        return string
    }

    public var debugDescription: String {
        var string = description
        if let response = result.value?.response, let debugStringConvertible = response as? CustomDebugStringConvertible {
            string += "\n\(debugStringConvertible.debugDescription)"
        }
        return string
    }
}
